package cn.wise.sc.pay.common.mq.server.service;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.common.mq.server.model.JMSDetail;
import org.springframework.amqp.core.AmqpAdmin;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Exchange;
import org.springframework.amqp.core.ExchangeBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.QueueBuilder;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class Producer implements IProducer {

    private final RabbitTemplate rabbitTemplate;

    private final AmqpAdmin amqpAdmin;

    public Producer(AmqpAdmin amqpAdmin, RabbitTemplate rabbitTemplate) {
        this.amqpAdmin = amqpAdmin;
        this.rabbitTemplate = rabbitTemplate;
    }

    /**
     * 发送消息
     *
     * @param routeKey 路由键
     * @param msg
     */
    @Override
    public void send(String routeKey, String msg) {
        this.rabbitTemplate.convertAndSend(routeKey, msg);
    }

    /**
     * 构建JMS协议绑定
     *
     * @param jmsDetail
     * @return
     */
    @Override
    public boolean bindJMS(JMSDetail jmsDetail) {

        if (check(jmsDetail)) {
            //构建队列
            Queue queue = QueueBuilder.durable(jmsDetail.getQueueName()).build();
            //构建交换器
            Exchange exchange = createExchange(jmsDetail);
            //绑定3者关系
            Binding binding = new Binding(jmsDetail.getQueueName(),
                    Binding.DestinationType.QUEUE, jmsDetail.getExchange(),
                    jmsDetail.getRouteKey(), null);
            try {
                //构建
                amqpAdmin.declareQueue(queue);
                amqpAdmin.declareExchange(exchange);
                amqpAdmin.declareBinding(binding);
                return true;
            } catch (Exception ex) {
                throw new RuntimeException("队列创建失败");
            }
        }
        return false;
    }

    private boolean check(JMSDetail jmsDetail) {

        return !StrUtil.isNotEmpty(jmsDetail.getQueueName()) &&
                !StrUtil.isEmpty(jmsDetail.getRouteKey()) &&
                !StrUtil.isEmpty(jmsDetail.getExchange());
    }

    private Exchange createExchange(JMSDetail jmsDetail){
        if (JMSDetail.ExchangType.FANOUT == jmsDetail.getExchangType()) {
           return ExchangeBuilder.fanoutExchange(jmsDetail.getExchange()).build();
        } else if (JMSDetail.ExchangType.TOPIC == jmsDetail.getExchangType()) {
            return ExchangeBuilder.topicExchange(jmsDetail.getExchange()).build();
        } else if (JMSDetail.ExchangType.HEADERS == jmsDetail.getExchangType()) {
            return ExchangeBuilder.headersExchange(jmsDetail.getExchange()).build();
        } else {
            return ExchangeBuilder.directExchange(jmsDetail.getExchange()).build();
        }
    }
}