package cn.wise.sc.pay.business.eshopping.utils;

import cn.wise.sc.pay.domain.eshopping.model.vo.GoodsSpecExcel;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import static java.sql.Types.NUMERIC;

public class ExcelReader {

    private static Logger logger = Logger.getLogger(ExcelReader.class.getName()); // 日志打印类

    private static final String XLS = "xls";
    private static final String XLSX = "xlsx";

    /**
     * 根据文件后缀名类型获取对应的工作簿对象
     * @param inputStream 读取文件的输入流
     * @param fileType 文件后缀名类型（xls或xlsx）
     * @return 包含文件数据的工作簿对象
     * @throws IOException
     */
    public static Workbook getWorkbook(InputStream inputStream, String fileType) throws IOException {
        Workbook workbook = null;
        if (fileType.equalsIgnoreCase(XLS)) {
            workbook = new HSSFWorkbook(inputStream);
        } else if (fileType.equalsIgnoreCase(XLSX)) {
            workbook = new XSSFWorkbook(inputStream);
        }
        return workbook;
    }

    /**
     * 读取Excel文件内容
     * @param  file 接口上传的文件
     * @return 读取结果列表，读取失败时返回null
     */
    public static List<GoodsSpecExcel> readExcel(MultipartFile file) {

        Workbook workbook = null;
        InputStream inputStream = null;

        try {
            String fileName = file.getOriginalFilename();
            // 获取Excel后缀名
            String fileType = fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length());
            // 获取Excel工作簿
            inputStream = file.getInputStream();
            workbook = getWorkbook(inputStream, fileType);

            // 读取excel中的数据
            List<GoodsSpecExcel> resultDataList = parseExcel(workbook);

            return resultDataList;
        } catch (Exception e) {
            return null;
        } finally {
            try {
                if (null != workbook) {
                    workbook.close();
                }
                if (null != inputStream) {
                    inputStream.close();
                }
            } catch (Exception e) {
                logger.warning("关闭数据流出错！错误信息：" + e.getMessage());
                return null;
            }
        }
    }

    /**
     * 解析Excel数据
     * @param workbook Excel工作簿对象
     * @return 解析结果
     */
    private static List<GoodsSpecExcel> parseExcel(Workbook workbook) {
        List<GoodsSpecExcel> resultDataList = new ArrayList<>();
        // 解析sheet
        for (int sheetNum = 0; sheetNum < workbook.getNumberOfSheets(); sheetNum++) {
            Sheet sheet = workbook.getSheetAt(sheetNum);

            // 校验sheet是否合法
            if (sheet == null) {
                continue;
            }

            // 获取第一行数据
            int firstRowNum = sheet.getFirstRowNum();
            Row firstRow = sheet.getRow(firstRowNum);
            if (null == firstRow) {
                logger.warning("解析Excel失败，在第一行没有读取到任何数据！");
            }

            // 解析每一行的数据，构造数据对象
            int rowStart = firstRowNum + 1;
            int rowEnd = sheet.getPhysicalNumberOfRows();
            for (int rowNum = rowStart; rowNum < rowEnd; rowNum++) {
                Row row = sheet.getRow(rowNum);
                if (null == row) {
                    continue;
                }
                GoodsSpecExcel resultData = convertRowToData(row);
                if (null == resultData) {
                    logger.warning("第 " + row.getRowNum() + "行数据不合法，已忽略！");
                    continue;
                }
                resultDataList.add(resultData);
            }
        }
        return resultDataList;
    }

    /**
     * 提取每一行中需要的数据，构造成为一个结果数据对象
     *
     * 当该行中有单元格的数据为空或不合法时，忽略该行的数据
     *
     * @param row 行数据
     * @return 解析后的行数据对象，行数据错误时返回null
     */
    private static GoodsSpecExcel convertRowToData(Row row) {
        GoodsSpecExcel resultData = new GoodsSpecExcel();

        Cell cell;
        int cellNum = 0;
        // 获取姓名
        cell = row.getCell(0);
        resultData.setGoodsName(cell.getStringCellValue());
        cell = row.getCell(1);
        resultData.setBrandName(cell.getStringCellValue());
        cell = row.getCell(2);
        resultData.setCategoryName(cell.getStringCellValue());
        cell = row.getCell(3);
        resultData.setWeight(cell.getNumericCellValue());
        cell = row.getCell(4);
        resultData.setIntro(cell.getStringCellValue());
        cell = row.getCell(5);
        resultData.setPrice(cell.getNumericCellValue());
        cell = row.getCell(6);
        resultData.setCost(cell.getNumericCellValue());
        cell = row.getCell(7);
        resultData.setMktprice(cell.getNumericCellValue());
        cell = row.getCell(8);
        resultData.setHavaSpec(cell.getStringCellValue());
        cell = row.getCell(9);
        resultData.setQuantity((int) cell.getNumericCellValue());
        cell = row.getCell(10);
        resultData.setHavaSpec(cell.getStringCellValue());
        cell = row.getCell(11);
        resultData.setGoodsTransfeeCharge(cell.getStringCellValue());
        cell = row.getCell(12);
        resultData.setTemplateName(cell.getStringCellValue());
        cell = row.getCell(13);
        resultData.setName(cell.getStringCellValue());
        cell = row.getCell(14);
        resultData.setDescr(cell.getStringCellValue());
        cell = row.getCell(15);
        resultData.setSpecPrice(cell.getNumericCellValue());
        cell = row.getCell(16);
        resultData.setStore((int) cell.getNumericCellValue());
        cell = row.getCell(17);
        resultData.setSpecWeight(cell.getNumericCellValue());
        cell = row.getCell(18);
        System.out.println(cell);
        return resultData;
    }
}