package cn.wise.sc.pay.business.exhibition.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.wise.sc.pay.business.exhibition.mapper.ExhibitionStoreMapper;
import cn.wise.sc.pay.business.exhibition.service.IExhibitionStoreService;
import cn.wise.sc.pay.business.exhibition.service.ITicketService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.redis.config.lock.RedissLockUtil;
import cn.wise.sc.pay.domain.exhibition.bean.ExhibitionStore;
import cn.wise.sc.pay.domain.exhibition.bean.Ticket;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-11-18
 */
@Service
@Slf4j
public class ExhibitionStoreServiceImpl extends ServiceImpl<ExhibitionStoreMapper, ExhibitionStore> implements IExhibitionStoreService {

    @Autowired
    ExhibitionStoreMapper exhibitionStoreMapper;
    @Autowired
    ITicketService iTicketService;
    @Autowired
    RedissonClient redissonClient;

    private final String EXHIBITION_STORE_KEY_PRE = "EXHIBITION_STORE_KEY_";
    private final String EXHIBITION_STORE_KEY_LOCK_PRE = "EXHIBITION_STORE_KEY_LOCK_";

    @Override
    public ExhibitionStore getStoreByTicketId(Long ticketId) {

        if (ticketId == null || ticketId <= 0) {
            throw new IllegalArgumentException("票型id为空!");
        }

        RAtomicLong store = redissonClient.getAtomicLong(EXHIBITION_STORE_KEY_PRE + ticketId);
        //能做redis走redis 不能就走数据库
        if (store.get() != 0) {
            ExhibitionStore eshopStore = new ExhibitionStore();
            eshopStore.setRemaining((int) store.get());
            eshopStore.setTicketId(ticketId);
            return eshopStore;
        } else {
            ExhibitionStore exhibitionStore = getGoodsStoreByGoodsId(ticketId);
            if (exhibitionStore != null) {
                store.set(exhibitionStore.getRemaining());
            }
            return exhibitionStore;
        }
    }

    @Override
    public BaseResponse<ExhibitionStore> newStore(Long ticketId, Integer quantity) {

        if (ticketId == null || ticketId == 0 || quantity == null || quantity == 0) {
            throw new IllegalArgumentException("商品id无效或者库存数量无效！");
        }
        ExhibitionStore eshopStore = new ExhibitionStore();
        eshopStore.setTicketId(ticketId);
        eshopStore.setRemaining(quantity);
        eshopStore.setVersion(999);
        String key = EXHIBITION_STORE_KEY_PRE + ticketId;
        boolean result = this.save(eshopStore);
        if (result) {
            //添加成功 将库存同步到redis
            RAtomicLong rAtomicLong = redissonClient.getAtomicLong(key);
            rAtomicLong.set(quantity);
            return BaseResponse.okMsg("添加成功!", BaseResponse.class);
        } else {
            return BaseResponse.error("添加失败!", BaseResponse.class.getName());
        }
    }

    @Override
    public BaseResponse<ExhibitionStore> addStore(Long ticketId, Integer quantity) {

        if (ticketId == null || ticketId <= 0 || quantity == null || quantity <= 0) {
            throw new IllegalArgumentException("商品id无效或者添加的库存数量无效！");
        }
        //找到当前库存
        ExhibitionStore eshopStore;
        String key = EXHIBITION_STORE_KEY_PRE + ticketId;
        eshopStore = getGoodsStoreByGoodsId(ticketId);

        if (eshopStore == null) {
            return BaseResponse.error("请求确定当前商品是否存在", ExhibitionStore.class.getName());
        }
        //判断redis中是否有key
        RAtomicLong atomicLong = redissonClient.getAtomicLong(key);
        boolean keyIsExist = atomicLong.isExists();
        if (!keyIsExist) {
            atomicLong.set(eshopStore.getRemaining());
        }

        UpdateWrapper<ExhibitionStore> uw = new UpdateWrapper<>();
        uw.eq("version", eshopStore.getVersion());
        uw.eq("ticket_id",eshopStore.getTicketId());
        //读写锁  锁当前商品redis库存  将redis库存补到数据库相同
        boolean updateSuccess = RedissLockUtil.readWriteLock(EXHIBITION_STORE_KEY_LOCK_PRE + ticketId, arg -> {
            RAtomicLong atomicLongOnce = redissonClient.getAtomicLong(key);
            long storeValue = atomicLongOnce.get();
            int store;
            try {
                store = (int) storeValue;
                //更新数据库库存
                ExhibitionStore updateEShopStore = new ExhibitionStore();
                BeanUtil.copyProperties(eshopStore, updateEShopStore);
                updateEShopStore.setRemaining(store + quantity);
                //版本号+1
                updateEShopStore.setVersion(eshopStore.getVersion() + 1);
                boolean result = this.update(updateEShopStore, uw);
                if (result) {
                    //将redis库存 +quantity
                    atomicLongOnce.set(store + quantity);
                }
            } catch (NumberFormatException e) {
                //更新数据库库存
                ExhibitionStore updateEShopStore = new ExhibitionStore();
                BeanUtil.copyProperties(eshopStore, updateEShopStore);
                updateEShopStore.setRemaining(eshopStore.getRemaining() + quantity);
                //版本号+1
                updateEShopStore.setVersion(eshopStore.getVersion() + 1);
                boolean result = this.update(updateEShopStore, uw);
                if (result) {
                    //将库存同步到redis
                    atomicLongOnce.set(eshopStore.getRemaining() + quantity);
                }
                return false;
            }
            return true;
        });

        return updateSuccess ? BaseResponse.okMsg("库存增加成功!", ExhibitionStore.class) :
                BaseResponse.error("库存增加失败!", ExhibitionStore.class.getName());
    }

    @Override
    public BaseResponse<ExhibitionStore> decreaseStore(Long ticketId, Integer quantity) {

        if (ticketId == null || ticketId <= 0 || quantity == null || quantity <= 0) {
            throw new IllegalArgumentException("商品id无效或者减去的库存数量无效！");
        }
        //初始化redis的key值
        String key;
        String lockKey;
        key = EXHIBITION_STORE_KEY_PRE + ticketId;
        lockKey = EXHIBITION_STORE_KEY_LOCK_PRE + ticketId;

        //判断redis中是否有key
        RAtomicLong bucket = redissonClient.getAtomicLong(key);
        boolean keyIsExist = bucket.isExists();
        if (!keyIsExist) {
            ExhibitionStore eshopStore = getGoodsStoreByGoodsId(ticketId);
            if (eshopStore != null) {
                bucket.set(eshopStore.getRemaining());
            } else {
                return BaseResponse.error("请确定当前商品或规格是否存在!", ExhibitionStore.class.getName());
            }
        }

        //写锁 和 addStore()防止脏读
        Long store = RedissLockUtil.readWriteLock(lockKey, arg -> {
            boolean flag = true;
            long timelyStore = -1;
            //操作redis减库存
            while (flag) {
                timelyStore = bucket.get() - quantity;
                flag = !bucket.compareAndSet(bucket.get(), timelyStore);
            }

            return timelyStore;
        });

        //<0 表示库存不足
        if (store < 0) {
            //写锁 和 addStore()防止脏读
            RedissLockUtil.readWriteLock(lockKey, arg -> {
                //库存回退 此次操作失败
                bucket.getAndAdd(quantity);
                return null;
            });
            return BaseResponse.error("库存不足!", ExhibitionStore.class.getName());
        }
        //=0 表示买完了
        if (store == 0) {
            boolean flag = true;
            //同步数据库库存
            while (flag) {
                BaseResponse<ExhibitionStore> baseResponse = this.saveDB(ticketId);
                flag = baseResponse.code == 400;
            }
        }

        return BaseResponse.okMsg("库存扣减成功!", ExhibitionStore.class);
    }

    @Override
    public BaseResponse<ExhibitionStore> saveDB(Long ticketId) {


        if (ticketId == null || ticketId == 0) {
            throw new IllegalArgumentException("商品id,无效！");
        }

        //初始化key
        String key = EXHIBITION_STORE_KEY_PRE + ticketId;
        ExhibitionStore eshopStore = getGoodsStoreByGoodsId(ticketId);
        //获取当前商品信息
        Ticket goods = iTicketService.getById(ticketId);
        if (goods == null || eshopStore == null) {
            log.error("没找到当前商品  =====  商品id:{}", ticketId);
            throw new NullPointerException();
        }

        //获取redis中的剩余库存
        RAtomicLong atomicLong = redissonClient.getAtomicLong(key);
        boolean keyIsExist = atomicLong.isExists();
        if (keyIsExist) {
            long store = atomicLong.get();

            if (eshopStore.getRemaining() == 0 || (store == eshopStore.getRemaining())) {
                return BaseResponse.okMsg("同步成功!", ExhibitionStore.class);
            }
            //根据提供的版本号更新
            ExhibitionStore updateEShopStore = new ExhibitionStore();
            BeanUtil.copyProperties(eshopStore, updateEShopStore);
            updateEShopStore.setRemaining((int) store);
            updateEShopStore.setVersion(eshopStore.getVersion() + 1);
            UpdateWrapper<ExhibitionStore> uw = new UpdateWrapper<>();
            uw.eq("version", eshopStore.getVersion());
            uw.eq("ticket_id",eshopStore.getTicketId());
            boolean updateSuccess = this.update(updateEShopStore, uw);
            if (!updateSuccess) {
                return BaseResponse.error("同步失败!", ExhibitionStore.class.getName());
            }

            return BaseResponse.okMsg("同步成功!", ExhibitionStore.class);
        } else {
            redissonClient.getAtomicLong(key).set(eshopStore.getRemaining());
            return BaseResponse.okMsg("同步成功!", ExhibitionStore.class);
        }

    }

    /**
     * 根据商品id获取库存
     *
     * @param id 商品id
     * @return EshopStore
     */
    private ExhibitionStore getGoodsStoreByGoodsId(Long id) {

        if (id == null || id == 0) {
            throw new IllegalArgumentException("商品id无效!");
        }
        QueryWrapper<ExhibitionStore> qw = new QueryWrapper<>();
        qw.eq("ticket_id", id);
        return this.getOne(qw);
    }
}
