package cn.wise.sc.pay.business.order.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.order.mapper.OrderCallExceptionMapper;
import cn.wise.sc.pay.business.order.service.IOrderCallExceptionService;
import cn.wise.sc.pay.business.order.service.IOrdersService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.domain.order.OrderCallExStatusUtil;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderCallExceptionStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * @description: 第三方支付调用异常处理实现类
 * @author: qh
 * @create: 2020-01-13 15:20
 **/
@Service
public class OrderCallExceptionServiceImpl extends ServiceImpl<OrderCallExceptionMapper, OrderCallException> implements IOrderCallExceptionService {

    @Autowired
    IOrdersService iOrdersService;
    /**
     * 最大允许失败次数
     */
//    @Value("call.exception.max")
    Integer max_fail = 20;

    @Override
    public BaseResponse createExceptionInfo(String orderNo) {

        if (StrUtil.isEmpty(orderNo)) {
            return BaseResponse.error("订单号无效", OrderCallException.class.getName());
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq("order_no", orderNo);
        Orders orders = iOrdersService.getOne(qw);
        if (orders != null) {
            //只有退款和支付会存在调用失败，因此限定状态
            try {
                OrderCallException callException = OrderCallException.builder()
                        .orderNo(orders.getOrderNo())
                        .orderStatus(OrderCallExStatusUtil.getExceptionStatus(orders.getOrderStatus()).name())
                        .build();
                try {
                    this.save(callException);
                    return BaseResponse.ok("添加成功", OrderCallException.class.getName());
                } catch (Exception ex) {
                    log.error("数据库添加失败------>IOrderCallExceptionService.createExceptionInfo()");
                    log.error(ex.getMessage());
                }
            } catch (Exception e) {
                log.error("订单类型获取失败---->IOrderCallExceptionService.createExceptionInfo()");
                log.error("订单类型无效");
            }
        }
        return BaseResponse.error("无效订单", OrderCallException.class.getName());
    }

    @Override
    public BaseResponse getList(OrderCallExceptionStatus exceptionStatus) {

        QueryWrapper qw = new QueryWrapper();
        qw.eq("order_status", exceptionStatus.name());
        qw.lt("fail_count", max_fail);
        qw.eq("status", 0);

        try {
            List<OrderCallException> list = this.list(qw);
            return BaseResponse.ok(list, OrderCallException.class);
        } catch (Exception ex) {
            log.error("数据库查询失败------>IOrderCallExceptionService.getList()");
            log.error(ex.getMessage());
        }
        return BaseResponse.error("查询失败", OrderCallException.class.getName());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse editBatchById(List<OrderCallException> list) {

        if (list == null || list.size() == 0) {
            return BaseResponse.error("参数为空", OrderCallException.class.getName());
        }
        List<String> unOrders = new ArrayList<>();
        for (OrderCallException callException : list) {
            if (callException.getFailCount() >= max_fail){
                callException.setStatus(1);
                if (OrderCallExceptionStatus.PAY_EXCEPTION.name()
                        .equals(callException.getOrderStatus())){
                    unOrders.add(callException.getOrderNo());
                }
            }

        }
        iOrdersService.cancelStatusByUnOrderNo(unOrders);
        try {
            this.updateBatchById(list);

            return BaseResponse.ok("批量修改成功", OrderCallException.class.getName());
        } catch (Exception ex) {
            log.error("数据库批量修改失败----->IOrderCallExceptionService.editBatchById()");
            log.error(ex.getMessage());
        }
        return BaseResponse.error("批量编辑失败", OrderCallException.class.getName());
    }

    @Override
    public BaseResponse createExceptionInfoBatch(List<OrderCallException> orderCallExceptions) {

        if (orderCallExceptions == null || orderCallExceptions.size() == 0) {
            return BaseResponse.error("参数为空", OrderCallException.class.getName());
        }

        //过滤掉已经存在的
        List<OrderCallException> list = this.list();
        List<OrderCallException> collect = orderCallExceptions.stream().filter(arg -> {
            AtomicBoolean flag = new AtomicBoolean(false);
            list.forEach(opt -> {
                if (opt.getOrderNo().equals(arg.getOrderNo())) {
                    flag.set(true);
                }
            });
            return !flag.get();
        }).collect(Collectors.toList());

        if (collect.size() != 0) {
            try {
                this.saveBatch(orderCallExceptions);
            } catch (Exception ex) {
                log.error("数据库批量添加失败----->IOrderCallExceptionService.createExceptionInfoBatch()");
                log.error(ex.getMessage());
                return BaseResponse.error("批量添加失败", OrderCallException.class.getName());
            }
        }
        return BaseResponse.ok("批量添加成功", OrderCallException.class.getName());
    }
}
