package cn.wise.sc.pay.business.order.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.wise.sc.pay.business.order.consumer.model.EshopGoodsMq;
import cn.wise.sc.pay.business.order.consumer.model.ExhibitionMq;
import cn.wise.sc.pay.business.order.consumer.model.GoodsMq;
import cn.wise.sc.pay.business.order.exception.ParameterException;
import cn.wise.sc.pay.business.order.mapper.OrdersMapper;
import cn.wise.sc.pay.business.order.service.IOrdersService;
import cn.wise.sc.pay.business.order.service.IXpenseTrackerService;
import cn.wise.sc.pay.business.order.service.Producer;
import cn.wise.sc.pay.business.order.service.strategy.IOrderStatisticsStrategy;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageWrapperQuery;
import cn.wise.sc.pay.common.core.query.PageWrapperQueryUtil;
import cn.wise.sc.pay.common.mq.client.TransactionMqRemoteClient;
import cn.wise.sc.pay.domain.order.model.OrderEventResult;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.model.StatisticsGoods;
import cn.wise.sc.pay.domain.order.model.XpenseTracker;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 订单实体类 服务实现类
 * </p>
 *
 * @author qh
 * @since 2019-12-12
 */
@Component
public class OrdersServiceImpl extends ServiceImpl<OrdersMapper, Orders> implements IService<Orders>, IOrdersService {

    final
    TransactionMqRemoteClient transactionMqRemoteClient;
    final
    Producer producer;
    final
    IXpenseTrackerService iXpenseTrackerService;

    public OrdersServiceImpl(TransactionMqRemoteClient transactionMqRemoteClient,
                             Producer producer, IXpenseTrackerService iXpenseTrackerService) {
        this.transactionMqRemoteClient = transactionMqRemoteClient;
        this.producer = producer;
        this.iXpenseTrackerService = iXpenseTrackerService;
    }

    @Override
    public boolean cancelStatusByUnOrderNo(List<String> unOrderNo) {

        if (unOrderNo==null || unOrderNo.size() == 0){
            return false;
        }
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("unified_order_no", unOrderNo);
        List<Orders> list = this.list(queryWrapper);
        List<Orders> updateOrders = new ArrayList<>(list.size());
        list.forEach(arg -> {
           Orders orders = new Orders();
           orders.setId(arg.getId());
           orders.setOrderStatus(OrderStatus.CANCELLED.name());
           updateOrders.add(orders);
        });

        return this.updateBatchById(updateOrders);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public OrderResponse createOrder(OrderDetailQuery orderDetail) {

        boolean isVaild = checkOrder(orderDetail);

        OrderResponse ordersEventResult;

        if (isVaild) {
            //生成订单对象
            Orders order = Orders.builder()
                    .orderNo(orderDetail.getOrderNo()) //订单号
                    .goods(JSON.toJSONString(orderDetail.getGoods())) //商品信息
                    .createTime(System.currentTimeMillis()) //订单生成时间
                    .totalPrice(BigDecimal.valueOf(orderDetail.getTotalPrice()).setScale(2, RoundingMode.HALF_UP)) //订单总金额
                    .remark(orderDetail.getRemark())
                    .sellerId(orderDetail.getSellerId()) //商户id
                    .userId(orderDetail.getUserId()) //买家id
                    .userPhone(orderDetail.getUserPhone()) //买家电话号码
                    .orderStatus(orderDetail.getStatus()) //订单初始化状态
                    .payStatus(PayStatus.PAY_ON.name()) //初始化支付状态为未支付
                    .payPrice(BigDecimal.valueOf(orderDetail.getPayPrice()).setScale(2, RoundingMode.HALF_UP))
                    .paas(orderDetail.getPaas()) //初始化平台
                    .paymentMethodName(orderDetail.getPayment()) //初始化支付方式
                    .goodsName(orderDetail.getGoodsName()) //初始化商品名字 冗余字段
                    .receivePlace(orderDetail.getAddress())
                    .freight(orderDetail.getFreight())
                    .build();
            //如果订单号为空时初始化订单号
            if (StrUtil.isEmpty(orderDetail.getOrderNo())) {
                order.setOrderNo(getPreOrderNo());
            }
            //桌号id
            if (StrUtil.isNotEmpty(orderDetail.getTableId())) {
                order.setTableId(orderDetail.getTableId());
            }
            //绑定设备号
            if (StrUtil.isNotEmpty(orderDetail.getDeviceNo())) {
                order.setDeviceNo(orderDetail.getDeviceNo());
            }
            //判断金额合法性 总-优惠-运费=支付
            if (new BigDecimal(orderDetail.getTotalPrice())
                    .subtract(orderDetail.getDiscount())
                    .subtract(orderDetail.getFreight())
                    .setScale(2, RoundingMode.HALF_DOWN)
                    .doubleValue()
                    != orderDetail.getPayPrice()) {
                return OrderResponse.error("创建订单失败,金额不匹配!");
            }
            if (StrUtil.isEmpty(orderDetail.getStatus())) {
                order.setOrderStatus(OrderStatus.WAIT_PAY.name());
            }
            try {
                this.save(order); //持久化订单信息
                ordersEventResult = OrderResponse.ok("订单创建成功", order);  //推事件
            } catch (Exception ex) {
                ordersEventResult = OrderResponse.error("订单创建失败");
                ordersEventResult.setCode(400);
            }

            return ordersEventResult;
        } else {
            return OrderResponse.error("订单创建失败");
        }
    }

    @Transactional
    public String cancle(OrderDetailQuery orderQuery) {

        return "回退";
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public OrderResponse editOrder(OrderDetailQuery orderQuery) {

        OrderResponse ordersEventResult = new OrderResponse();
        if (StrUtil.isEmpty(orderQuery.getOrderNo())) {
            return OrderResponse.error("参数错误,订单为空!");
        }

        Orders order = Orders.builder()
                .updateTime(System.currentTimeMillis()) //维护订单更新时间
                .remark(orderQuery.getRemark()) //订单备注
                .sellerId(orderQuery.getSellerId()) //卖家id
                .userId(orderQuery.getUserId()) //买家id
                .userPhone(orderQuery.getUserPhone()) //买家手机号
                .paas(orderQuery.getPaas())
                .orderStatus(orderQuery.getStatus()) //维护订单状态
                .feild1("6") //保留字段
                .feild2(orderQuery.getLogisticsNo())//设置物流
                .payStatus(orderQuery.getPayStatus()) //支付状态
                .paymentMethodName(orderQuery.getPayment()) //支付方式
                .build();

        //当下单号 不为空时  更新下单号儿
        if (StrUtil.isNotEmpty(orderQuery.getUnifiedOrderNo())) {
            order.setUnifiedOrderNo(orderQuery.getUnifiedOrderNo());
        }
        if (orderQuery.getGoods() != null) {
            order.setGoods(JSON.toJSONString(orderQuery.getGoods()));
        }
        if (orderQuery.getTotalPrice() != 0d) {
            order.setTotalPrice(new BigDecimal(orderQuery.getTotalPrice()));
        }
        if (orderQuery.getPayPrice() != 0d) {
            order.setPayPrice(new BigDecimal(orderQuery.getPayPrice()));
        }
        if (StrUtil.isNotEmpty(orderQuery.getPayStatus())) {
            order.setPayStatus(orderQuery.getPayStatus());
        }
        if (StrUtil.isNotBlank(orderQuery.getRefund())) {
            order.setCashier(orderQuery.getRefund());
        }

        QueryWrapper qw = new QueryWrapper();
        qw.eq("order_no", orderQuery.getOrderNo());
        try {
            this.update(order, qw); //持久化订单对象
            ordersEventResult.setData(order);
            ordersEventResult.setMsg("订单更新成功!");
            ordersEventResult.setCode(200);
        } catch (Exception ex) {
            ordersEventResult.setData(null);
            ordersEventResult.setMsg("订单更新失败！");
            ordersEventResult.setCode(400);
        }

        return ordersEventResult;
    }

    @Override
    public OrderResponse changeOrderStatus(String orderNo, OrderStatus
            orderStatus, Map<String, Object> condition) {

        UpdateWrapper uw = new UpdateWrapper();

        if (StrUtil.isEmpty(orderNo)) {
            throw new ParameterException("订单号不能为空!");
        }
        if (orderStatus == null) {
            throw new ParameterException("订单状态不能为空!");
        }

        if (condition != null && condition.size() != 0) {
            for (String key : condition.keySet()) {
                uw.eq(key, condition.get(key));
            }
        }

        uw.eq("order_no", orderNo);
        uw.set("order_status", orderStatus.name());

        boolean isSuccess = this.update(uw);

        if (isSuccess) {
            //如果订单状态为完成  向消费记录mq发消息
            if (OrderStatus.COMPLETE.name().equals(orderStatus.name())) {
                QueryWrapper qw = new QueryWrapper();
                qw.eq("order_no", orderNo);
                Orders orders = this.getOne(qw);
                orderCompleteEvent(orders);
            }
            return OrderResponse.ok("状态改变成功");
        }
        return OrderResponse.error("状态改变失败");
    }

    @Override
    public OrderResponse changeOrderStatusByUserId(String orderNo, String userId, OrderStatus
            orderStatus, Map<String, Object> condition) {

        UpdateWrapper uw = new UpdateWrapper();

        if (StrUtil.isEmpty(orderNo)) {
            throw new ParameterException("订单号不能为空!");
        }
        if (orderStatus == null) {
            throw new ParameterException("订单状态不能为空!");
        }
        if (StrUtil.isEmpty(userId)) {
            if (condition != null && condition.size() != 0) {
                for (String key : condition.keySet()) {
                    uw.eq(key, condition.get(key));
                }
            }
        }
        uw.eq("order_no", orderNo);
        uw.set("order_status", orderStatus.name());
        uw.set("user_id", userId);
        boolean isSuccess = this.update(uw);

        if (isSuccess) {
            return OrderResponse.ok("状态改变成功");
        }
        return OrderResponse.error("状态改变失败");
    }

    @Override
    public BaseResponse countOrdersByDate(Long start, Long end, OrderStatus status, String sellerId) {

        if (BeanUtil.isEmpty(start) || BeanUtil.isEmpty(end) || start.longValue() >= end) {
            return OrderResponse.error("无效的时间参数");
        }

        QueryWrapper qw = Wrappers.query();
        qw.between("create_time", start, end);

        if (status != null) {
            qw.eq("order_status", status.name());
        }

        if (sellerId != null) {
            qw.eq("seller_id", sellerId);
        }
        try {
            int count = this.count(qw);
            BaseResponse baseResponse = BaseResponse.ok(count);
            baseResponse.setClassName(Orders.class.getName());
            return baseResponse;
        } catch (Exception ex) {
            return OrderResponse.error("查询失败");
        }
    }

    @Override
    public OrderResponse getOrders(Map<String, Object> condition) {

        QueryWrapper uw = new QueryWrapper();
        uw.orderByDesc("create_time");
        if (condition != null && condition.size() != 0) {
            for (String key : condition.keySet()) {
                if (condition.get(key).equals(OrderStatus.ALL.name())) {
                    continue;
                }
                uw.eq(key, condition.get(key));
            }
        }
        try {
            List<Orders> orders = this.list(uw);
            OrderResponse orderResponse = OrderResponse.ok(orders);
            return orderResponse;
        } catch (Exception ex) {
            return OrderResponse.error("查询异常");
        }

    }

    @Override
    public OrderResponse getOrdersByUserId(String userId, OrderStatus status) {

        QueryWrapper qw = Wrappers.<Orders>query();

        if (StrUtil.isEmpty(userId)) {
            return OrderResponse.error("userId必传");
        }
        qw.eq("user_id", userId);
        qw.orderByDesc("create_time");
        if (!OrderStatus.ALL.name().equals(status.name())) {
            qw.eq("order_status", status.name());
        }
        try {

            List<Orders> list = this.list(qw);
            return OrderResponse.ok(list);
        } catch (Exception ex) {
            return OrderResponse.error("查询失败");
        }
    }

    @Override
    public OrderResponse delOrders(String orderNo, Map<String, Object> condition) {

        QueryWrapper uw = new QueryWrapper();

        if (StrUtil.isEmpty(orderNo)) {
            throw new ParameterException("订单号不能为空!");
        }
        uw.eq("order_no", orderNo);
        if (condition != null && condition.size() != 0) {
            for (String key : condition.keySet()) {
                uw.eq(key, condition.get(key));
            }
        }
        try {
            this.remove(uw);
            return OrderResponse.ok("删除成功");
        } catch (Exception ex) {
            return OrderResponse.error("删除失败");
        }
    }

    @Override
    public OrderResponse cancelOrder(String orderNo) {

        if (StrUtil.isEmpty(orderNo)) {
            throw new ParameterException("订单号不能为空!");
        }

        UpdateWrapper uw = new UpdateWrapper();

        uw.eq("order_no", orderNo);

        uw.set("order_status", OrderStatus.CANCELLED.name());
        try {
            this.update(uw);
            return OrderResponse.ok("取消订单成功");
        } catch (Exception ex) {
            return OrderResponse.error("取消订单失败");
        }
    }

    @Override
    public OrderResponse getOneOrder(Map<String, Object> condition) {
        QueryWrapper uw = new QueryWrapper();

        if (condition != null && condition.size() != 0) {
            for (String key : condition.keySet()) {
                uw.eq(key, condition.get(key));
            }
        }

        try {
            Orders orders = this.getOne(uw);
            return OrderResponse.ok("获取订单成功", orders);
        } catch (Exception ex) {
            return OrderResponse.error("获取失败");
        }
    }

    @Override
    public String getPreOrderNo() {

        Snowflake snowflake = IdUtil.createSnowflake(1, 1);
        String orderNo = snowflake.nextIdStr();

        return orderNo;
    }


    @Override
    public long getCount(QueryWrapper ew) {

        return super.count(ew);
    }

    @Override
    public OrderResponse page(PageWrapperQuery pageWrapperQuery) {

        IPage ipage = new Page(pageWrapperQuery.getPageQuery().getPageNo(), pageWrapperQuery.getPageQuery().getPageSize());
        QueryWrapper queryWrapper = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        queryWrapper.orderByDesc("create_time");
        try {

            Page<Orders> page = (Page<Orders>) this.page(ipage, queryWrapper);
            OrderResponse orderResponse = OrderResponse.ok("分页查询成功");
            orderResponse.setPage(page);
            return orderResponse;
        } catch (Exception ex) {
            OrderResponse orderResponse = OrderResponse.error("分页查询失败");
            return orderResponse;
        }

    }

    @Override
    public boolean updateBatch(List<OrderDetailQuery> ordersCancels) {
        if (ordersCancels == null || ordersCancels.size() == 0) {
            return true;
        }
        //2种方式更新  优先使用id 然后统一下单号
        List<Orders> ordersUpdate = new ArrayList<>(ordersCancels.size());
        //获取id不为空的
        List<OrderDetailQuery> collect1 = ordersCancels.stream()
                .filter(arg -> arg.getId() != 0L)
                .collect(Collectors.toList());
        for (OrderDetailQuery orderDetailQuery : collect1) {
            ordersUpdate.add(OrderDetailQuery2Orders(orderDetailQuery));
        }
        //获取统一下单号不为空的
        List<OrderDetailQuery> collect = ordersCancels.stream()
                .filter(arg -> StrUtil.isNotBlank(arg.getUnifiedOrderNo()))
                .collect(Collectors.toList());
        if (collect.size() != 0) {
            List<String> unifiedOrderNos = collect.stream()
                    .map(arg -> arg.getUnifiedOrderNo())
                    .collect(Collectors.toList());
            QueryWrapper<Orders> qw = new QueryWrapper<>();
            qw.in("unified_order_no", unifiedOrderNos);
            List<Orders> list = this.list(qw);
            //关联id
            for (OrderDetailQuery orderDetailQuery : collect) {
                Orders orders = list.stream()
                        .filter(arg -> orderDetailQuery.getUnifiedOrderNo().equals(arg.getUnifiedOrderNo()))
                        .findFirst().orElse(null);
                List<XpenseTracker> xpenseTrackers = getXpense(orders);
                iXpenseTrackerService.createBatch(xpenseTrackers);
                if (orders != null) {
                    orderDetailQuery.setId(orders.getId());
                    ordersUpdate.add(OrderDetailQuery2Orders(orderDetailQuery));
                }
            }
        }

        return this.updateBatchById(ordersUpdate);
    }

    @Override
    public BaseResponse statisticsTop(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end) {

        //金额统计
        BigDecimal tradeMoney = iOrderStatisticsStrategy.statisticsTradeMoney(shopName, start, end);
        BigDecimal refundMoney = iOrderStatisticsStrategy.statisticsRefundMoney(shopName, start, end);
        BigDecimal tradeSubRefundMoney = tradeMoney.subtract(refundMoney);

        //笔数统计
        Integer tradeCount = iOrderStatisticsStrategy.statisticsTradeCount(shopName, start, end);
        Integer refundCount = iOrderStatisticsStrategy.statisticsRefundCount(shopName, start, end);
        Map<String, Object> rts = new HashMap<>();
        rts.put("交易金额", tradeMoney.doubleValue());
        rts.put("交易笔数", tradeCount.intValue());
        rts.put("退款金额", refundMoney.doubleValue());
        rts.put("退款笔数", refundCount.intValue());
        rts.put("收入金额", tradeSubRefundMoney.doubleValue());
        BaseResponse baseResponse = BaseResponse.ok(rts);
        baseResponse.setClassName(Orders.class.getName());
        return baseResponse;
    }

    @Override
    public BaseResponse statisticsByChart(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end) {

        //交易折线图数据
        Map<String, BigDecimal> tradeMoneyChartData = iOrderStatisticsStrategy.statisticsTradeMoneyChart(shopName, start, end);
        Map<String, Integer> tradeCountChartData = iOrderStatisticsStrategy.statisticsTradeCountChart(shopName, start, end);

        //退款折线图数据
        Map<String, BigDecimal> refundMoneyChartData = iOrderStatisticsStrategy.statisticsRefundMoneyChart(shopName, start, end);
        Map<String, Integer> refundCountChartData = iOrderStatisticsStrategy.statisticsRefundCountChart(shopName, start, end);
        Map<String, Map> rts = new HashMap<>();

        rts.put("交易金额", tradeMoneyChartData);
        rts.put("交易笔数", tradeCountChartData);

        rts.put("退款金额", refundMoneyChartData);
        rts.put("退款笔数", refundCountChartData);
        BaseResponse baseResponse = BaseResponse.ok(rts);
        baseResponse.setClassName(Orders.class.getName());
        return baseResponse;
    }

    @Override
    public BaseResponse statisticsCountPayMethod(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end) {

        Map<String, Integer> countByPayMenthod = iOrderStatisticsStrategy.statisticsCountByPayMenthod(shopName, start, end);
        BaseResponse baseResponse = BaseResponse.ok(countByPayMenthod);
        baseResponse.setClassName(Orders.class.getName());
        return baseResponse;
    }

    @Override
    public BaseResponse statisticsMoneyPayMethod(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end) {

        Map<String, BigDecimal> moneyByPayMenthod = iOrderStatisticsStrategy.statisticsMoneyByPayMenthod(shopName, start, end);
        Map<String, Double> moneyByPayMenthodDo = new HashMap<>(moneyByPayMenthod.size());
        for (String key : moneyByPayMenthod.keySet()
        ) {
            if (StrUtil.isEmpty(key)) {
                continue;
            }
            moneyByPayMenthodDo.put(key, moneyByPayMenthod.get(key).doubleValue());
        }
        BaseResponse baseResponse = BaseResponse.ok(moneyByPayMenthodDo);
        baseResponse.setClassName(Orders.class.getName());
        return baseResponse;
    }

    @Override
    public BaseResponse statisticsList(IOrderStatisticsStrategy iOrderStatisticsStrategy, String shopName, long start, long end, int payment) {
        List<Orders> list = iOrderStatisticsStrategy.statisticsList(shopName, start, end, payment);
        BaseResponse baseResponse = BaseResponse.ok(new Orders());
        baseResponse.setList(list);
        return baseResponse;
    }

    @Override
    public OrderResponse getListcomplex(PageWrapperQuery pageWrapperQuery) {
        QueryWrapper queryWrapper = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        queryWrapper.orderByDesc("update_time");
        try {
            List<Orders> list = this.list(queryWrapper);
            return OrderResponse.ok(list);

        } catch (Exception ex) {
            return OrderResponse.error("查询失败");
        }
    }

    @Override
    public BaseResponse statisticsAggregate(PageWrapperQuery pageWrapperQuery) {

        QueryWrapper qw = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        qw.eq("pay_status", "PAY_YES");
        qw.eq("paas", "cashier");
        List<Orders> list = this.list(qw);

        //返回结果
        HashMap<String, BigDecimal> rts = MapUtil.newHashMap(2);
        rts.put("Aggregate", new BigDecimal(0));
        rts.put("Money", new BigDecimal(0));
        rts.put("Discounts", new BigDecimal(0));

        if (list.size() == 0) {
            return BaseResponse.ok(rts);
        }

        //统计聚合支付的金额
        List<Orders> aggregateList = list.stream()
                .filter(arg -> !"现金支付".equals(arg.getPaymentMethodName()))
                .collect(Collectors.toList());
        for (Orders order : aggregateList) {
            BigDecimal amount = order.getPayPrice();
            BigDecimal aggregateTotal = rts.get("Aggregate");
            rts.put("Aggregate", aggregateTotal.add(amount));
        }
        //统计现金支付的金额
        List<Orders> moneyList = list.stream()
                .filter(arg -> "现金支付".equals(arg.getPaymentMethodName()))
                .collect(Collectors.toList());
        for (Orders order : moneyList) {
            BigDecimal amount = order.getPayPrice();
            BigDecimal aggregateTotal = rts.get("Money");
            rts.put("Money", aggregateTotal.add(amount));
        }
        //统计优惠金额
        for (Orders order : list) {
            BigDecimal discountAmount = order.getDiscount();
            BigDecimal discountTotal = rts.get("Discounts");
            rts.put("Discounts", discountTotal.add(discountAmount));
        }

        return BaseResponse.ok(rts);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse<List<StatisticsGoods>> statisticsGoodsMarketMoneyGroupSellerGood(PageWrapperQuery pageWrapperQuery) {

        //获取当天所有订单
        QueryWrapper qw = PageWrapperQueryUtil.getQueryWrapper(pageWrapperQuery);
        qw.eq("paas", "cashier");
        qw.eq("pay_status", "PAY_YES");
        List<Orders> list = this.list(qw);

        //统计商品集合
        List<StatisticsGoods> rts = new ArrayList<>();
        for (Orders order : list) {
            String goodsStr = order.getGoods();
            List<StatisticsGoods> statisticsGoods = JSON.parseArray(goodsStr, StatisticsGoods.class);
            for (StatisticsGoods good : statisticsGoods) {
                good.setSellerId(order.getSellerId());
            }
            rts.addAll(statisticsGoods);
        }

        return BaseResponse.ok(rts);
    }

    /**
     * query转Orders实体
     *
     * @param ordersCancels
     * @return
     */
    private Orders OrderDetailQuery2Orders(OrderDetailQuery ordersCancels) {

        Orders orders = Orders.builder()
                .id(ordersCancels.getId())
                .orderStatus(ordersCancels.getStatus())
                .paas(ordersCancels.getPaas())
                .payStatus(ordersCancels.getPayStatus())
                .remark(ordersCancels.getRemark())
                .unifiedOrderNo(ordersCancels.getUnifiedOrderNo())
                .build();

        if (OrderStatus.WAIT_DELIVERING.name().equals(ordersCancels.getStatus())) {
            orders.setPayStatus(PayStatus.PAY_YES.name());
        }

        //值类型 判断是否为0,选择更新
        if (ordersCancels.getTotalPrice() != 0) {
            orders.setTotalPrice(new BigDecimal(ordersCancels.getTotalPrice()));
        }
        if (ordersCancels.getPayPrice() != 0) {
            orders.setPayPrice(new BigDecimal(ordersCancels.getPayPrice()));
        }
        return orders;
    }

    @Override
    public OrderResponse payOrder(String orderNo) {
        OrderEventResult.builder()
                .build();
        return null;
    }


    /**
     * 判断订单参数有效否
     *
     * @param orderQuery
     * @return
     */
    private boolean checkOrder(OrderDetailQuery orderQuery) {

        return orderQuery.getGoods().size() != 0
                && orderQuery.getSellerId() != 0
                && !(orderQuery.getTotalPrice() < 0);
    }

    /**
     * 获取map的valus
     *
     * @param map 目标map
     * @return list
     */
    private <T> List<T> getMapValues(Map<String, T> map) {

        List list = CollectionUtil.newArrayList();
        for (String key : map.keySet()) {
            list.add(map.get(key));
        }

        return list;
    }

    /**
     * 订单完成时 发布事件
     */
    private void orderCompleteEvent(Orders orders) {
        if (orders == null) {
            throw new RuntimeException();
        }
        String msg = JSON.toJSONString(orders);
        producer.send("create_xpense", msg);
    }

    /**
     * 根据订单平台解析商品信息
     *
     * @param order
     * @return
     */
    private List<XpenseTracker> getXpense(Orders order) {
        List<XpenseTracker> xpenseTrackers = new ArrayList<>();
        Assert.notNull(order, "订单为空");
        //订单来自展览
        if ("EXHIBITION".equals(order.getPaas())) {
            List<ExhibitionMq> list = JSON.parseArray(order.getGoods(), ExhibitionMq.class);
            for (ExhibitionMq ex : list
            ) {
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() + " — " + ex.getTitle())
                        .price(ex.getPrice())
                        .orderNo(order.getOrderNo())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getUpdateTime())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        //订单来自收银系统
        if ("cashier".equals(order.getPaas())) {
            List<GoodsMq> list = JSON.parseArray(order.getGoods(), GoodsMq.class);
            for (GoodsMq ex : list
            ) {
                if (ex.getPrice() == null || ex.getTotalPrice() == null) {
                    continue;
                }
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() + " — " + ex.getName())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getCreateTime())
                        .orderNo(order.getOrderNo())
                        .price(ex.getTotalPrice().doubleValue())
                        .num(ex.getBuyCount())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        if ("ESHOP".equals(order.getPaas())) {
            List<EshopGoodsMq> list = JSON.parseArray(order.getGoods(), EshopGoodsMq.class);
            for (EshopGoodsMq goods : list) {
                XpenseTracker xpenseTracker = XpenseTracker.builder()
                        .name(order.getPaymentMethodName() + " — " + goods.getGoodsName())
                        .userId(order.getUserId().longValue())
                        .createTime(order.getCreateTime())
                        .price(goods.getPrice())
                        .orderNo(order.getOrderNo())
                        .build();
                xpenseTrackers.add(xpenseTracker);
            }
        }
        return xpenseTrackers;
    }
}
