package cn.wise.sc.payment.api.service;

import cn.wise.sc.pay.common.core.model.BaseResponse;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * @description: 招行聚合支付 服务接口
 * 详细参考 : /wiki/招商银行商户聚合支付接口V2.1.doc
 * @author: qh
 * @create: 2020-01-14 09:30
 **/
@FeignClient(value = "saas-v1-payment", path = "/sms", fallback = ZHPaymentServiceImplHystrix.class)
public interface IZHPaymentService {

    /**
     * 小程序获取openid
     *
     * @param code
     * @return
     */
    @RequestMapping(value = "/getOpenid", method = RequestMethod.GET)
    ResponseEntity<Map<String, Object>> getUserOpenId(String code);

    /**
     * 微信统一下单
     *
     * @param request 请求报文  用于获取主机ip
     * @param orderId 订单号
     * @param openId  openId
     * @param body
     * @param txnAmt  金额   分为单位
     * @return
     */
    @PostMapping("/unifiedOrder")
    BaseResponse wxUnifiedOrder(HttpServletRequest request, @RequestParam("orderId") String orderId,
                                @RequestParam("openId") String openId, @RequestParam("body") String body,
                                @RequestParam("txnAmt") String txnAmt);

    /**
     * 支付主动查询
     *
     * @param cmbOrderId 招行聚合订单号 （订单号二选一,二者都传默认使用聚合订单号）
     * @param orderId    订单号
     * @return
     */
    @PostMapping("/payResultQuery")
    BaseResponse payResultQuery(@RequestParam("cmbOrderId") String cmbOrderId, @RequestParam("orderId") String orderId);

    /**
     * 支付结果通知
     *
     * @param cmbOrderId 招行聚合订单号 （订单号二选一,二者都传默认使用聚合订单号）
     * @param orderId    订单号
     * @param txnAmt     金额
     * @param dscAmt     优惠金额
     * @param payType    支付类型
     * @param txnTime    时间
     * @return
     */
    @PostMapping("payResultNotice")
    BaseResponse payResultNotice(@RequestParam("cmbOrderId") String cmbOrderId, @RequestParam("orderId") String orderId,
                                 @RequestParam("txnAmt") String txnAmt,
                                 @RequestParam("dscAmt") String dscAmt, @RequestParam("payType") String payType,
                                 @RequestParam("txnTime") String txnTime);

    /**
     * 退款主动查询
     *
     * @param orderId        订单号
     * @param origCmbOrderId 招行聚合订单号 （订单号二选一,二者都传默认使用聚合订单号）
     * @return
     */
    @PostMapping("payRefundQuery")
    BaseResponse payRefundQuery(@RequestParam("orderId") String orderId, @RequestParam("origCmbOrderId") String origCmbOrderId);

    /**
     * 申请退款
     *
     * @param orderId      退款订单号
     * @param origOrderId  支付订单号
     * @param txnAmt       金额
     * @param refundAmt    退款金额
     * @param refundReason 原因
     * @return
     */
    @PostMapping("applyForRefund")
    BaseResponse applyForRefund(@RequestParam("orderId") String orderId,
                                @RequestParam("origOrderId") String origOrderId,
                                @RequestParam("txnAmt") String txnAmt, @RequestParam("refundAmt") String refundAmt,
                                @RequestParam("refundReason") String refundReason);

    /**
     * 退款通知
     *
     * @param orderId      订单号
     * @param cmbOrderId   招行聚合订单号 （订单号二选一,二者都传默认使用聚合订单号）
     * @param refundAmt    退款金额
     * @param refundDscAmt
     * @param couponInfo
     * @param payType
     * @param txnTime
     * @param endDate
     * @param endTime
     * @param mchReserved
     * @return
     */
    @PostMapping("payRefundNotice")
    BaseResponse payRefundNotice(@RequestParam("orderId") String orderId, @RequestParam("cmbOrderId") String cmbOrderId,
                                 @RequestParam("refundAmt") String refundAmt, @RequestParam("refundDscAmt") String refundDscAmt,
                                 @RequestParam("couponInfo") String couponInfo, @RequestParam("payType") String payType,
                                 @RequestParam("txnTime") String txnTime, @RequestParam("endDate") String endDate,
                                 @RequestParam("endTime") String endTime, @RequestParam("mchReserved") String mchReserved);

    /**
     * 关闭订单
     *
     * @param origOrderId    订单
     * @param origCmbOrderId 招行聚合订单号 （订单号二选一,二者都传默认使用聚合订单号）
     * @return
     */
    @PostMapping("closeOrder")
    BaseResponse closeOrder(@RequestParam("origOrderId") String origOrderId, @RequestParam("origCmbOrderId") String origCmbOrderId);

    /**
     * 获取对账地址
     *
     * @param billDate 下载对账单的日期，格式：20180803
     * @return
     */
    @PostMapping("getBillUrl")
    BaseResponse getBillUrl(@RequestParam("billDate") String billDate);

    /**
     * 设置密钥
     *
     * @param platformId    招行分配的平台或技术服务商的编号
     * @param digEvpEncMth  数字信封加密算法 01：RSA2
     * @param digEvp        数字信封
     * @param encryptMethod 加密方法 02：AES
     * @param encryptInfo   encryptInfo AES256加密数据包
     * @param keyTpye       keyTpye RSA/SM2/MD5等，目前只支持RSA
     * @param keyLen        keyLen 1024/2048/256，目前只支持2048长度的公钥
     * @param mchKey        mchKey 商户公钥明文
     * @param updateUrl     保留使用
     * @return
     */
    @PostMapping("setSecretKey")
    BaseResponse setSecretKey(@RequestParam("platformId") String platformId, @RequestParam("digEvpEncMth") String digEvpEncMth,
                              @RequestParam("digEvp") String digEvp, @RequestParam("encryptMethod") String encryptMethod,
                              @RequestParam("encryptInfo") String encryptInfo, @RequestParam("keyTpye") String keyTpye,
                              @RequestParam("keyLen") String keyLen, @RequestParam("mchKey") String mchKey,
                              @RequestParam("updateUrl") String updateUrl);

    /**
     * 发起服务窗口支付
     *
     * @param deviceInfo         终端设备号
     * @param orderId            商户端生成，要求此订单号在整个商户下唯一
     * @param tradeScene         OFFLINE:线下场景(OFFLINE合同)
     *                           INSURANCE：保险
     *                           CHARITY：公益
     * @param body               商品描述
     * @param mchReserved        商户保留域
     * @param timeoutExpress     支付有效时间
     * @param notifyUrl          交易通知地址
     * @param txnAmt             交易金额
     * @param currencyCode       交易币种
     * @param disablePayChannels 限制支付渠道
     * @param buyerLogonId       去买家支付宝账号
     * @param buyerId            买家支付宝用户ID
     * @param identity           实名支付信息
     * @param policyNo           保单单号
     * @param region             地区码
     * @return
     */
    @PostMapping("servicePay")
    BaseResponse servicePay(@RequestParam("deviceInfo") String deviceInfo, @RequestParam("orderId") String orderId,
                            @RequestParam("tradeScene") String tradeScene, @RequestParam("body") String body,
                            @RequestParam("mchReserved") String mchReserved, @RequestParam("timeoutExpress") String timeoutExpress,
                            @RequestParam("notifyUrl") String notifyUrl, @RequestParam("txnAmt") String txnAmt,
                            @RequestParam("currencyCode") String currencyCode, @RequestParam("disablePayChannels") String disablePayChannels,
                            @RequestParam("buyerLogonId") String buyerLogonId, @RequestParam("buyerId") String buyerId,
                            @RequestParam("identity") String identity, @RequestParam("policyNo") String policyNo,
                            @RequestParam("region") String region);
}
