package cn.wise.sc.pay.business.points.service.impl;

import cn.wise.sc.pay.business.points.FastDFSUtils;
import cn.wise.sc.pay.business.points.mapper.CouponTypeMapper;
import cn.wise.sc.pay.business.points.service.ICouponTypeService;
import cn.wise.sc.pay.business.points.service.ICouponsService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.query.PageQuery;
import cn.wise.sc.pay.domain.point.bean.CouponType;
import cn.wise.sc.pay.domain.point.query.CouponTypeQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 优惠券类型业务实现类
 *
 * @author jiawei
 */
@Service
public class CouponTypeServiceImpl extends ServiceImpl<CouponTypeMapper, CouponType> implements ICouponTypeService {
    @Autowired
    FastDFSUtils fastDFSUtils;
    @Autowired
    ICouponsService iCouponsService;

    @Override
    /**
     * 添加一条优惠券记录
     */
    public BaseResponse add(CouponTypeQuery couponTypeQuery) {
        if (StringUtils.isBlank(couponTypeQuery.getName())) {
            return BaseResponse.error("请传入类型名称", CouponTypeServiceImpl.class.getName());
        }
        CouponType couponType = CouponTypeQuery.query2Entity(couponTypeQuery);
        //默认状态
        if (couponTypeQuery.getStatus() == null) {
            couponType.setStatus(0);
        }
        couponType.setCreateTime(System.currentTimeMillis());

        QueryWrapper qw = new QueryWrapper();
        qw.eq("name", couponType.getName());
        qw.eq("del_flag", 1);
        int count = this.count(qw);
        if (count > 0) {
            return BaseResponse.error("当前名字的类型已存在", this.getClass().getName());
        }

        boolean save = this.save(couponType);
        return save ? BaseResponse.ok(couponType) : BaseResponse.error("添加数据失败", CouponTypeServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据类型名称条件分页
     */
    public BaseResponse pageOnCondition(String name, PageQuery pageQuery) {
        Page<CouponType> page = new Page<>();
        page.setCurrent(pageQuery.getPageNo());
        page.setSize(pageQuery.getPageSize());

        QueryWrapper<CouponType> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("create_time");

        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("name", name);
        }
        IPage<CouponType> page1 = this.page(page, queryWrapper);
        //图片拼接
        if (page1 != null || !CollectionUtils.isEmpty(page1.getRecords())) {
            page1.getRecords().stream().forEach(ct -> ct.setImage(fastDFSUtils.fullUrl(ct.getImage())));
        }
        return BaseResponse.ok(page1);
    }

    @Override
    /**
     * 根据id禁用  或者根据id批量禁用
     */
    public BaseResponse disable(List<Integer> ids) {
        if (ids == null || ids.size() < 1) {
            return BaseResponse.error("请传入id值", CouponTypeServiceImpl.class.getName());
        }

        //判断优惠卷是否被使用
        QueryWrapper qw = new QueryWrapper();
        qw.in("type_id", ids);
        int count = iCouponsService.count(qw);
        if (count > 0){
            return BaseResponse.error("优惠卷类型被使用中!",this.getClass().getName());
        }

        UpdateWrapper<CouponType> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", ids);
        updateWrapper.set("status", 1);
        boolean update = this.update(updateWrapper);
        return update ? BaseResponse.ok() : BaseResponse.error("禁用失败", CouponTypeServiceImpl.class.getName());
    }


    @Override
    /**
     * 根据id启用  或者根据id批量启用
     */
    public BaseResponse enable(List<Integer> ids) {
        if (ids == null || ids.size() < 1) {
            return BaseResponse.error("请传入id值", CouponTypeServiceImpl.class.getName());
        }
        UpdateWrapper<CouponType> updateWrapper = new UpdateWrapper<>();
        updateWrapper.in("id", ids);
        updateWrapper.set("status", 0);
        boolean update = this.update(updateWrapper);
        return update ? BaseResponse.ok() : BaseResponse.error("启用失败", CouponTypeServiceImpl.class.getName());
    }

    @Override
    /**
     * 修改数据信息，传入数据时请传入id值
     */
    public BaseResponse edit(CouponTypeQuery couponTypeQuery) {
        if (couponTypeQuery.getId() == null) {
            return BaseResponse.error("请传入id值", CouponTypeServiceImpl.class.getName());
        }
        CouponType couponType = CouponTypeQuery.query2Entity(couponTypeQuery);
        couponType.setUpdateTime(System.currentTimeMillis());
        boolean update = this.updateById(couponType);
        return update ? BaseResponse.ok(couponType) : BaseResponse.error("更新信息失败", CouponTypeServiceImpl.class.getName());
    }

    @Override
    /**
     * 根据id值删除记录
     */
    public BaseResponse discard(Integer id) {
        if (id == null) {
            return BaseResponse.error("请传入id值", CouponTypeServiceImpl.class.getName());
        }

        //判断优惠卷是否被使用
        QueryWrapper qw = new QueryWrapper();
        qw.eq("type_id", id);
        int count = iCouponsService.count(qw);
        if (count > 0){
            return BaseResponse.error("优惠卷类型被使用中!",this.getClass().getName());
        }

        boolean remove = this.removeById(id);
        return remove ? BaseResponse.ok() : BaseResponse.error("删除信息失败", CouponTypeServiceImpl.class.getName());
    }

    @Override
    public BaseResponse infoById(Long id) {
        if (id == null) {
            return BaseResponse.error("请传入id值", CouponsServiceImpl.class.getName());
        }
        CouponType info = this.getById(id);
        //图片拼接
        if (info != null) {
            info.setImage(fastDFSUtils.fullUrl(info.getImage()));
        }
        return BaseResponse.ok(info, CouponsServiceImpl.class);
    }

    @Override
    /**
     * 列表显示可用优惠券类型
     */
    public BaseResponse listAvailable() {
        QueryWrapper<CouponType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", 0);
        List<CouponType> couponTypes = this.list(queryWrapper);
        //图片拼接
        if (!CollectionUtils.isEmpty(couponTypes)) {
            couponTypes.stream().forEach(ct -> ct.setImage(fastDFSUtils.fullUrl(ct.getImage())));
        }
        BaseResponse<List<CouponType>> baseResponse = BaseResponse.ok(couponTypes);
        return baseResponse;
    }
}
