package cn.wise.sc.pay.task.timer.order.handler;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.wise.sc.pay.api.order.service.IOrderCallExService;
import cn.wise.sc.pay.api.order.service.IOrdersService;
import cn.wise.sc.pay.api.order.service.IZHPaymentService;
import cn.wise.sc.pay.common.core.model.BaseResponse;
import cn.wise.sc.pay.common.core.model.TransactionMessage;
import cn.wise.sc.pay.domain.order.model.OrderCallException;
import cn.wise.sc.pay.domain.order.model.OrderResponse;
import cn.wise.sc.pay.domain.order.model.OrderStatus;
import cn.wise.sc.pay.domain.order.model.Orders;
import cn.wise.sc.pay.domain.order.model.PayStatus;
import cn.wise.sc.pay.domain.order.query.OrderDetailQuery;
import cn.wise.sc.pay.task.timer.order.config.Producer;
import cn.wise.sc.pay.task.timer.order.model.PayRtsQueryParam;
import cn.wise.sc.pay.task.timer.order.model.PayRtsTradeState;
import com.alibaba.fastjson.JSON;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ConcurrentLinkedDeque;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * @description: 订单调用招行支付回调异常 发起主动查询 定时处理器
 * @author: qh
 * @create: 2020-01-13 16:51
 **/
@Slf4j
@Component
public class OrderPayCallExHandler {

    @Autowired
    IOrderCallExService orderCallExService;
    @Autowired
    IZHPaymentService izhPaymentService;
    @Autowired
    IOrdersService orderService;
    @Autowired
    Producer producer;


    /**
     * 轮询支付失败的订单  主动发起支付结果查询
     */
    @XxlJob("OrderCallExceptionHandler")
    public ReturnT<String> execute(String s) {

        BaseResponse orderResponse = orderCallExService.getList(0);
        if (orderResponse.getCode() == 400) {
            return ReturnT.FAIL;
        }
        List<HashMap> hashMaps = orderResponse.getList();
        //HashMap转Bean
        List<OrderCallException> payExceptionList = hashMaps.stream()
                .map(arg -> BeanUtil.mapToBean(arg, OrderCallException.class, true))
                .collect(Collectors.toList());
        /*
         * 轮询  支付结果返回异常的订单，发起主动查询
         *       成功->修改订单状态为待发货,关闭支付异常状态
         *       失败->修改订单为已取消,关闭支付异常状态
         *       交易进行->不处理,失败处理次数+1
         *       转入退款-->修改订单状态为退款,关闭异常状态
         * */
        List<OrderDetailQuery> editOrders = new ArrayList<>();
        List<OrderCallException> editOrderCallExList = new ArrayList<>(payExceptionList.size());

        for (OrderCallException orderCallEx : payExceptionList) {

            //找原始订单号  orderCallEx的orderNo存的是统一下单号
            Map condition = new HashMap(1);
            condition.put("unified_order_no", orderCallEx.getOrderNo());
            Orders orders = orderService.getOneOrder(JSON.toJSONString(condition)).getData();
            if (orders == null) {
                continue;
            }
            BaseResponse payResponse = izhPaymentService.payResultQuery("",
                    orderCallEx.getOrderNo(), orders.getSellerId() + "");
            if (payResponse.getCode() == 400) {
                continue;
            }
            OrderCallException editOrderCallEx = new OrderCallException();
            BeanUtil.copyProperties(orderCallEx, editOrderCallEx);
            if (payResponse.getCode() == 200) {
                PayRtsQueryParam payRtsQueryParam = JSON.parseObject(payResponse.getData().toString(), PayRtsQueryParam.class);
                OrderDetailQuery editOrder = new OrderDetailQuery();
                editOrder.setUnifiedOrderNo(orderCallEx.getOrderNo());
                if (PayRtsTradeState.S.name().equals(payRtsQueryParam.getTradeState())) {
                    //已支付
                    editOrder.setStatus(OrderStatus.WAIT_DELIVERING.name());
                    editOrder.setPayStatus(PayStatus.PAY_YES.name());
                    editOrders.add(editOrder);
                    editOrderCallEx.setStatus(1);
                    //推送小票消息
                    pushReceiptMsg(orderCallEx.getOrderNo());
                } else if (PayRtsTradeState.P.name().equals(payRtsQueryParam.getTradeState())) {
                    //待支付
                    editOrderCallEx.setFailCount(orderCallEx.getFailCount() + 1);
                } else {
                    editOrder.setStatus(OrderStatus.CANCELLED.name());
                    editOrders.add(editOrder);
                    editOrderCallEx.setStatus(1);
                }
            }else {
                //待支付
                editOrderCallEx.setFailCount(orderCallEx.getFailCount() + 1);
            }
            editOrderCallExList.add(editOrderCallEx);
        }

        BaseResponse baseResponse = BaseResponse.ok("");

        if (editOrderCallExList.size() != 0) {
            baseResponse = orderCallExService.editBatchById(editOrderCallExList);
        }
        if (editOrders.size() != 0 && baseResponse.getCode() == 200) {
            baseResponse = orderService.updateBatch(editOrders);
        }
        if (baseResponse.getCode() == 200) {
            return ReturnT.SUCCESS;
        } else {
            return ReturnT.FAIL;
        }
    }

    static ThreadFactory factory = r -> new Thread("订单支付回调异常处理线程");
    static ThreadPoolExecutor executor = new ThreadPoolExecutor(50,
            200, 10, TimeUnit.SECONDS, new ArrayBlockingQueue<>(100),
            factory, new ThreadPoolExecutor.AbortPolicy());

    @XxlJob("OrderCallExceptionHandlerAsync")
    public ReturnT<String> executeAsync(String s) {

        /*
         * 根据seller_id 对不同的订单处理
         * */
        BaseResponse orderResponse = orderCallExService.getList(0);
        if (orderResponse.getCode() == 400) {
            return ReturnT.FAIL;
        }
        List<HashMap> hashMaps = orderResponse.getList();
        //HashMap转Bean
        List<OrderCallException> payExceptionList = hashMaps.stream()
                .map(arg -> BeanUtil.mapToBean(arg, OrderCallException.class, true))
                .collect(Collectors.toList());

        /*
         * 如果有100个商户,开100个线程？明显不对！！
         * 怎么复用。
         * */
        Set<Long> idsSet = payExceptionList.stream().map(OrderCallException::getSellerId).collect(Collectors.toSet());
        ConcurrentLinkedDeque<Long> orderCallExceptionIds = new ConcurrentLinkedDeque<>(idsSet);


        AtomicBoolean flag = new AtomicBoolean(true);
        //为每个商户开启支付异常处理查询
        for (int i = 0; i < idsSet.size(); i++) {
            //开启线程
            executor.execute(() -> {
                //如果处理完了 不再处理
                if (orderCallExceptionIds.isEmpty()) {
                    return;
                }
                //弹出第一个商户id  处理当前id下的所有订单
                Long sellerId = orderCallExceptionIds.poll();
                List<OrderCallException> orderCallExceptions = payExceptionList.stream()
                        .filter(arg -> arg.getSellerId().longValue() == sellerId)
                        .collect(Collectors.toList());
                List<OrderDetailQuery> editOrders = new ArrayList<>();
                List<OrderCallException> editOrderCallExList = new ArrayList<>(payExceptionList.size());
                //对每笔订单发起主动查询
                orderCallExceptions.forEach(arg -> {
                    BaseResponse payResponse = izhPaymentService.payResultQuery("",
                            arg.getOrderNo(), sellerId + "");
                    OrderCallException editOrderCallEx = new OrderCallException();
                    BeanUtil.copyProperties(arg, editOrderCallEx);
                    if (payResponse.getCode() == 200) {
                        PayRtsQueryParam payRtsQueryParam = JSON.parseObject(payResponse.getData().toString(), PayRtsQueryParam.class);
                        OrderDetailQuery editOrder = new OrderDetailQuery();
                        editOrder.setUnifiedOrderNo(arg.getOrderNo());
                        if (PayRtsTradeState.S.name().equals(payRtsQueryParam.getTradeState())) {
                            //已支付
                            editOrder.setStatus(OrderStatus.WAIT_DELIVERING.name());
                            editOrder.setPayStatus(PayStatus.PAY_YES.name());
                            editOrders.add(editOrder);
                            editOrderCallEx.setStatus(1);
                            //推送小票消息
                            pushReceiptMsg(arg.getOrderNo());
                            //todo 推积分

                        } else if (PayRtsTradeState.P.name().equals(payRtsQueryParam.getTradeState())) {
                            //待支付
                            editOrderCallEx.setFailCount(arg.getFailCount() + 1);
                        } else {
                            editOrder.setStatus(OrderStatus.CANCELLED.name());
                            editOrders.add(editOrder);
                            editOrderCallEx.setStatus(1);
                        }
                    } else {
                        //待支付
                        editOrderCallEx.setFailCount(arg.getFailCount() + 1);
                    }
                    editOrderCallExList.add(editOrderCallEx);
                });
                BaseResponse baseResponse = BaseResponse.ok("");
                if (editOrderCallExList.size() != 0) {
                    baseResponse = orderCallExService.editBatchById(editOrderCallExList);
                    if (baseResponse.getCode() == 200) {
                        log.info("当前商户< {} >,支付回调异常表更新成功!", sellerId);
                    } else {
                        log.error("当前商户< {} >,支付回调异常表更新成功!", sellerId);
                        flag.set(false);
                    }
                }
                if (editOrders.size() != 0 && baseResponse.getCode() == 200) {
                    baseResponse = orderService.updateBatch(editOrders);
                    if (baseResponse.getCode() == 200) {
                        log.info("当前商户< {} >,主动查询后,更新订单状态成功!", sellerId);
                    } else {
                        log.error("当前商户< {} >,主动查询后,更新订单状态失败!", sellerId);
                        flag.set(false);
                    }
                }
            });
        }
        if (flag.get()) {
            return ReturnT.SUCCESS;
        } else {
            return ReturnT.FAIL;
        }
    }

    /**
     * 推送打印小票的消息
     *
     * @param unOrderNo
     */
    private void pushReceiptMsg(String unOrderNo) {

        //构建msg对象
        Map<String, Object> condition = new HashedMap(1);
        condition.put("unified_order_no", unOrderNo);
        OrderResponse oneOrder = orderService.getOneOrder(JSON.toJSONString(condition));
        Orders orders = oneOrder.getCode() == 200 ? oneOrder.getData() : null;
        if (orders == null) {
            oneOrder = orderService.getOneOrder(JSON.toJSONString(condition));
            orders = oneOrder.getCode() == 200 ? oneOrder.getData() : null;
        }
        //收银系统 打印小票
        if ("cashier".equals(orders.getPaas()) && orders != null) {
            String msg = "主动查询单子 " + orders.getPaymentMethodName() + orders.getPayPrice() + "元 " +
                    DateUtil.format(new Date(orders.getUpdateTime()), "yyyy-MM-dd hh:mm:ss") + "|" + orders.getOrderNo();

            String sn = orders.getDeviceNo();
            TransactionMessage transactionMessage = new TransactionMessage();
            transactionMessage.setMessage(msg);
            transactionMessage.setQueue(sn);
            transactionMessage.setRouteKey(sn);
            try {
                mqpost(transactionMessage);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public OrderResponse mqpost(TransactionMessage msg) throws IOException {
        msg.setCreateTime(LocalDateTime.now());
        producer.createQueue(msg.getQueue());
        System.out.println(msg.getQueue());
        System.out.println(msg.getRouteKey());
        producer.send(msg.getRouteKey(), JSON.toJSONString(msg));
        return OrderResponse.ok("发送成功");
    }
}
